# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

modules = \
['alianator']
install_requires = \
['titlecase>=2.3,<3.0']

setup_kwargs = {
    'name': 'alianator',
    'version': '2.0.0',
    'description': 'alianator is a tool that helps Pycord and discord.py users easily resolve user-facing aliases for Discord permission flags.',
    'long_description': '# alianator\n\n<a href="https://pypi.org/project/alianator/"><img src="https://img.shields.io/pypi/pyversions/alianator?logo=python&logoColor=white&style=for-the-badge"></a>\n<a href="https://pypi.org/project/alianator/"><img src="https://img.shields.io/pypi/v/alianator?color=green&logo=pypi&logoColor=white&style=for-the-badge"></a>\n<a href="https://github.com/celsiusnarhwal/alianator/releases"><img src="https://img.shields.io/github/v/release/celsiusnarhwal/alianator?color=orange&label=latest%20release&logo=github&sort=semver&style=for-the-badge"></a>\n<a href="https://github.com/celsiusnarhwal/alianator/blob/master/LICENSE.md"><img src="https://img.shields.io/pypi/l/alianator?color=03cb98&style=for-the-badge"></a>\n\nalianator is a tool that helps [Pycord](https://github.com/Pycord-Development/pycord) and\n[discord.py](https://github.com/Rapptz/discord.py) users easily resolve user-facing aliases for Discord permission\nflags.\n\n## Installation\n\n```bash\n$ pip install alianator\n```\n\nalianator doesn\'t include either Pycord or discord.py as a dependency; instead, it allows you to use whichever of the \ntwo libraries you prefer. alianator **does not and will not** support other Discord API wrappers, such as\n[Nextcord](https://github.com/nextcord/nextcord), [Hikari](https://github.com/hikari-py/hikari), or\n[disnake](https://github.com/DisnakeDev/disnake).\n\n## Usage\n\nalianator can resolve aliases from `discord.Permissions` objects, integers, strings, tuples, lists of strings, and lists\nof tuples.\n\n```python\nimport alianator\n\nalianator.resolve(arg, mode=mode)\n```\n\nThe optional `mode` flag can be used to specify which permissions should be resolved. If `mode` is `True`, only granted\npermissions will be resolved; if `mode` is `False`, only denied permissions will be resolved; if `mode` is `None`, all\npermissions will be resolved. If `mode` is not explicitly specified, it will default to `True`.\n\n```python\nimport alianator\nimport discord\n\n# Resolving from a discord.Permissions object\nperms = discord.Permissions.general()\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Manage Channels\', \'Manage Server\', \'View Audit Log\', \'Read Messages\', \'View Guild Insights\', \'Manage Roles\', \'Manage Webhooks\', \'Manage Emojis and Stickers\']\n\n\n# Resolving from an integer\nperms = 3072\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Read Messages\', \'Send Messages\']\n\n\n# Resolving from a string\nperms = "send_tts_messages"\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Send Text-To-Speech Messages\']\n\n\n# Resolving from a tuple\nperms = ("moderate_members", True)\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Timeout Members\']\n\n\n# Resolving from a list of strings\nperms = ["manage_guild", "manage_emojis"]\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Manage Server\', \'Manage Emojis and Stickers\']\n\n\n# Resolving from a list of tuples\nperms = [("use_slash_commands", True), ("use_voice_activation", True)]\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Use Application Commands\', \'Use Voice Activity\']\n```\n\nThat\'s about all there is to it. alianator does one thing and does it well.\n\n## License\n\nalianator is released under the [MIT License](https://github.com/celsiusnarhwal/alianator/blob/master/LICENSE.md).\n',
    'author': 'celsius narhwal',
    'author_email': 'celsiusnarhwal@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/celsiusnarhwal/alianator',
    'package_dir': package_dir,
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

import sys
from titlecase import titlecase
from typing import Union

try:
    import discord
except ImportError as err:
    err.msg = "alianator couldn't find Pycord or discord.py in your environment. Install one (and only one!) of " \
              "them, then try again."
    print(err, file=sys.stderr)
    exit(1)


def resolve(arg: Union[discord.Permissions, int, str, tuple, list[str], list[tuple]], mode: bool = True) -> list[str]:
    """
    Resolves Discord permission names from their API flags to their user-facing aliases.

    :param arg: A discord.Permissions object, a integer, string tuple, list of strings, or list of tuples
    representing the permissions to resolve.

    :param mode: A boolean flag that determines which permissions will be resolved. If True, only granted
    permissions will be resolved. If False, only denied permissions will be resolved. If None, all permissions
    will be resolved. Defaults to True. If the function recieves a string or list of strings, all permissions will
    be resolved regardless of the value of this flag.

    :return: A list of strings containing the resolved permission aliases.
    """
    def resolver(names: list[str]) -> list[str]:
        resolutions = {
            "external_emojis": "Use External Emoji",
            "external_stickers": "Use External Stickers",
            "manage_emojis": "Manage Emojis and Stickers",
            "manage_guild": "Manage Server",
            "mention_everyone": "Mention \\@everyone, \\@here, and All Roles",
            "moderate_members": "Timeout Members",
            "send_tts_messages": "Send Text-to-Speech Messages",
            "start_embedded_activities": "Use Activities",
            "stream": "Video",
            "use_slash_commands": "Use Application Commands",
            "use_voice_activation": "Use Voice Activity",
        }

        return [resolutions.get(name) or titlecase(name.replace("_", " ")) for name in names]

    def validator(candidate: Union[tuple, str]) -> bool:
        if isinstance(candidate, tuple):
            return validator(candidate[0]) and isinstance(candidate[1], bool)
        elif isinstance(candidate, str):
            return candidate in discord.Permissions.VALID_FLAGS

    if not isinstance(mode, Union[bool, None]):
        raise TypeError("mode must be a boolean or None.")

    if isinstance(arg, list):
        if all(isinstance(x, str) for x in arg):
            if any(not validator(x) for x in arg):
                raise ValueError("List contains invalid permission flags")
            else:
                raws = arg

        elif all(isinstance(x, tuple) for x in arg):
            if any(not validator(x) for x in arg):
                raise TypeError("All permissions must be tuples of the format (str, bool) where str is a valid "
                                "permission flag.")
            else:
                raws = [x[0] for x in arg] if mode is None else [x[0] for x in arg if x[1] is mode]

        else:
            raise TypeError("If you pass in a list, it must be comprised of either exclusively strings or exclusively "
                            "tuples of the format (str, bool) where str is a valid permission flag.")

    elif isinstance(arg, str):
        if validator(arg):
            raws = [arg]
        else:
            raise ValueError(f"{arg} is not a valid permission flag.")

    elif isinstance(arg, tuple):
        if validator(arg):
            raws = [arg[0]] if mode is None else [arg[0]] if arg[1] is mode else []
        else:
            raise TypeError("If you pass in a tuple, it must be of the format (str, bool) where str is a valid "
                            "permission flag.")

    elif isinstance(arg, int):
        raws = [p[0] for p in discord.Permissions(arg)] if mode is None else [p[0] for p in discord.Permissions(arg)
                                                                              if p[1] is mode]

    elif isinstance(arg, discord.Permissions):
        raws = [p[0] for p in arg] if mode is None else [p[0] for p in arg if p[1] is mode]

    else:
        raise TypeError("Argument must be a discord.Permissions object, an integer, a string, a tuple, or a list.")

    return resolver(raws)
